@@ -1,5 +1,5 @@
 #include <Core/ModelicaDefine.h>
- #include <Core/Modelica.h>
+#include <Core/Modelica.h>
 #include <Solver/IDA/IDA.h>
 #include <Core/Math/Functions.h>
 
@@ -10,10 +10,19 @@ Ida::Ida(IMixedSystem* system, ISolverSettings* settings)
     : SolverDefaultImplementation(system, settings),
       _idasettings(dynamic_cast<ISolverSettings*>(_settings)),
       _idaMem(NULL),
-      _z(NULL),
+      /*_z(NULL),
       _zInit(NULL),
       _zWrite(NULL),
+	  */
+	  _y(NULL),
+	  _yp(NULL),
+      _yInit(NULL),
+      _yWrite(NULL),
+	  _ypWrite(NULL),
+	  _dae_res(NULL),
       _dimSys(0),
+	  _dimAE(0),
+	  _dimStates(0),
       _cv_rt(0),
       _outStps(0),
       _locStps(0),
@@ -85,16 +94,31 @@ Ida::Ida(IMixedSystem* system, ISolverSettings* settings)
 
 Ida::~Ida()
 {
+  /*
   if (_z)
     delete[] _z;
   if (_zInit)
     delete[] _zInit;
+  if (_zWrite)
+      delete[] _zWrite;
+*/
+  if (_y)
+    delete[] _y;
+  if (_yp)
+    delete[] _yp;
+  if (_yInit)
+    delete[] _yInit;
+  if (_yWrite)
+      delete[] _yWrite;
+  if (_ypWrite)
+      delete[] _ypWrite;
+  if (_dae_res)
+	  delete[] _dae_res;
   if (_zeroSign)
     delete[] _zeroSign;
   if (_absTol)
     delete[] _absTol;
-  if (_zWrite)
-      delete[] _zWrite;
+
   if (_ida_initialized)
   {
     N_VDestroy_Serial(_CV_y0);
@@ -141,10 +165,16 @@ void Ida::initialize()
   _tLastEvent = 0.0;
   _event_n = 0;
   SolverDefaultImplementation::initialize();
-  _dimSys = _continuous_system->getDimContinuousStates();
-  _dimZeroFunc = _event_system->getDimZeroFunc();
 
-  if (_dimSys <= 0)
+  _dimStates = _continuous_system->getDimContinuousStates();
+  _dimZeroFunc = _event_system->getDimZeroFunc();
+  _dimAE = _continuous_system->getDimAE();
+   if(_dimAE>0)
+		_dimSys=_dimAE+ _dimStates;
+	else
+		_dimSys=_dimStates;
+  if (_dimStates <= 0)
+
   {
     _idid = -1;
     throw std::invalid_argument("Ida::initialize()");
@@ -152,36 +182,60 @@ void Ida::initialize()
   else
   {
     // Allocate state vectors, stages and temporary arrays
-    if (_z)
+
+   /*if (_z)
       delete[] _z;
     if (_zInit)
       delete[] _zInit;
     if (_zWrite)
-      delete[] _zWrite;
+      delete[] _zWrite;*/
+    if (_y)
+      delete[] _y;
+    if (_yInit)
+      delete[] _yInit;
+    if (_yWrite)
+      delete[] _yWrite;
+    if (_ypWrite)
+      delete[] _ypWrite;
+    if (_yp)
+      delete[] _yp;
+    if (_dae_res)
+      delete[] _dae_res;
     if (_zeroSign)
       delete[] _zeroSign;
     if (_absTol)
       delete[] _absTol;
-  if(_delta)
-    delete [] _delta;
+    if(_delta)
+      delete [] _delta;
     if(_deltaInv)
-    delete [] _deltaInv;
+      delete [] _deltaInv;
     if(_ysave)
-    delete [] _ysave;
+      delete [] _ysave;
 
-    _z = new double[_dimSys];
+
+	_y = new double[_dimSys];
+	_yp = new double[_dimSys];
+    _yInit = new double[_dimSys];
+    _yWrite = new double[_dimSys];
+	_ypWrite = new double[_dimSys];
+	_dae_res = new double[_dimSys];
+	/*
+	_z = new double[_dimSys];
     _zInit = new double[_dimSys];
     _zWrite = new double[_dimSys];
+	*/
+
     _zeroSign = new int[_dimZeroFunc];
     _absTol = new double[_dimSys];
     _delta =new double[_dimSys];
     _deltaInv =new double[_dimSys];
     _ysave =new double[_dimSys];
 
-    memset(_z, 0, _dimSys * sizeof(double));
-    memset(_zInit, 0, _dimSys * sizeof(double));
-  memset(_ysave, 0, _dimSys * sizeof(double));
-
+    memset(_y, 0, _dimSys * sizeof(double));
+	memset(_yp, 0, _dimSys * sizeof(double));
+    memset(_yInit, 0, _dimSys * sizeof(double));
+    memset(_ysave, 0, _dimSys * sizeof(double));
+	 std::fill_n(_absTol, _dimSys, 1.0);
     // Counter initialisieren
     _outStps = 0;
 
@@ -205,19 +259,25 @@ void Ida::initialize()
     //
 
     // Set initial values for IDA
-    _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
-    _continuous_system->getContinuousStates(_zInit);
-    memcpy(_z, _zInit, _dimSys * sizeof(double));
-
+    //_continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+   _continuous_system->getContinuousStates(_yInit);
+    memcpy(_y, _yInit, _dimStates * sizeof(double));
+    if(_dimAE>0)
+	{
+       _mixed_system->getAlgebraicDAEVars(_yInit+_dimStates);
+	    memcpy(_y+_dimStates, _yInit+_dimStates, _dimAE * sizeof(double));
+	  _continuous_system->getContinuousStates(_yp);
+	}
     // Get nominal values
-    _continuous_system->getNominalStates(_absTol);
-    for (int i = 0; i < _dimSys; i++)
-      _absTol[i] *= dynamic_cast<ISolverSettings*>(_idasettings)->getATol();
+	 _continuous_system->getNominalStates(_absTol);
+    for (int i = 0; i < _dimStates; i++)
+	    _absTol[i] = dynamic_cast<ISolverSettings*>(_idasettings)->getATol();
 
-    _CV_y0 = N_VMake_Serial(_dimSys, _zInit);
-    _CV_y = N_VMake_Serial(_dimSys, _z);
-    _CV_yp = N_VNew_Serial(_dimSys);
-    _CV_yWrite = N_VMake_Serial(_dimSys, _zWrite);
+    _CV_y0 = N_VMake_Serial(_dimSys, _yInit);
+    _CV_y = N_VMake_Serial(_dimSys, _y);
+    _CV_yp = N_VMake_Serial(_dimSys, _yp);
+    _CV_yWrite = N_VMake_Serial(_dimSys, _yWrite);
+	_CV_ypWrite = N_VMake_Serial(_dimSys, _ypWrite);
     _CV_absTol = N_VMake_Serial(_dimSys, _absTol);
 
     if (check_flag((void*) _CV_y0, "N_VMake_Serial", 0))
@@ -225,15 +285,19 @@ void Ida::initialize()
       _idid = -5;
       throw std::invalid_argument("Ida::initialize()");
     }
-    calcFunction(_tCurrent, NV_DATA_S(_CV_y0), NV_DATA_S(_CV_yp));
+
+	//is already initialized: calcFunction(_tCurrent, NV_DATA_S(_CV_y0), NV_DATA_S(_CV_yp),NV_DATA_S(_CV_yp));
+
     // Initialize Ida (Initial values are required)
-    _idid = IDAInit(_idaMem, CV_fCallback, _tCurrent, _CV_y0, _CV_yp);
+    _idid = IDAInit(_idaMem, rhsFunctionCB, _tCurrent, _CV_y0, _CV_yp);
     if (_idid < 0)
     {
       _idid = -5;
       throw std::invalid_argument("Ida::initialize()");
     }
-
+	_idid = IDASetErrHandlerFn(_idaMem, errOutputIDA, _data);
+	 if (_idid < 0)
+      throw std::invalid_argument("IDA::initialize()");
     // Set Tolerances
     _idid = IDASVtolerances(_idaMem, dynamic_cast<ISolverSettings*>(_idasettings)->getRTol(), _CV_absTol);    // RTOL and ATOL
     if (_idid < 0)
@@ -265,18 +329,29 @@ void Ida::initialize()
       throw std::invalid_argument(/*_idid,_tCurrent,*/"IDA::initialize()");
 
     // Initialize linear solver
-  _idid = IDADense(_idaMem, _dimSys);
+    _idid = IDADense(_idaMem, _dimSys);
     if (_idid < 0)
       throw std::invalid_argument("IDA::initialize()");
+    if(_dimAE>0)
+	{
+	    _idid = IDASetSuppressAlg(_idaMem, TRUE);
+        double* tmp = new double[_dimSys];
+	    std::fill_n(tmp, _dimStates, 1.0);
+	    std::fill_n(tmp+_dimStates, _dimAE, 0.0);
+	   _idid = IDASetId(_idaMem, N_VMake_Serial(_dimSys,tmp));
+	    delete [] tmp;
+	    if (_idid < 0)
+         throw std::invalid_argument("IDA::initialize()");
+	}
 
   // Use own jacobian matrix
-  //_idid = CVDlsSetDenseJacFn(_idaMem, &CV_JCallback);
+  //_idid = CVDlsSetDenseJacFn(_idaMem, &jacobianFunctionCB);
   //if (_idid < 0)
   //    throw std::invalid_argument("IDA::initialize()");
 
     if (_dimZeroFunc)
     {
-      _idid = IDARootInit(_idaMem, _dimZeroFunc, &CV_ZerofCallback);
+      _idid = IDARootInit(_idaMem, _dimZeroFunc, &zeroFunctionCB);
 
       memset(_zeroSign, 0, _dimZeroFunc * sizeof(int));
       _idid = IDASetRootDirection(_idaMem, _zeroSign);
@@ -287,7 +362,7 @@ void Ida::initialize()
 
     }
 
-    initializeColoredJac();
+
     _ida_initialized = true;
 
     //
@@ -353,7 +428,7 @@ void Ida::solve(const SOLVERCALL action)
         writeToFile(0, _tCurrent, _h);
       if (writeOutput)
         writeIDAOutput(_tCurrent, _h, _locStps);
-    _continuous_system->getContinuousStates(_z);
+       _continuous_system->getContinuousStates(_y);
     }
 
     // Solver soll fortfahren
@@ -483,7 +558,7 @@ void Ida::IDACore()
     // Perform state selection
     bool state_selection = stateSelection();
     if (state_selection)
-      _continuous_system->getContinuousStates(_z);
+      _continuous_system->getContinuousStates(_y);
 
     _zeroFound = false;
 
@@ -535,7 +610,14 @@ void Ida::IDACore()
       // Write the values of (P1)
       if (writeEventOutput)
       {
-        _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+		if(_dimAE>0)
+		{
+			 _continuous_system->evaluateDAE(IContinuous::CONTINUOUS);
+        }
+		else
+		{
+		    _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+		}
         writeToFile(0, _tCurrent, _h);
       }
 
@@ -548,8 +630,13 @@ void Ida::IDACore()
       {
         // State variables were reinitialized, thus we have to give these values to the ida-solver
         // Take care about the memory regions, _z is the same like _CV_y
-        _continuous_system->getContinuousStates(_z);
-  calcFunction(_tCurrent, NV_DATA_S(_CV_y), NV_DATA_S(_CV_yp));
+        _continuous_system->getContinuousStates(_y);
+        if(_dimAE>0)
+		{
+           _mixed_system->getAlgebraicDAEVars(_y+_dimStates);
+		   _continuous_system->getRHS(_yp);
+		}
+		calcFunction(_tCurrent, NV_DATA_S(_CV_y), NV_DATA_S(_CV_yp),_dae_res);
 
       }
     }
@@ -560,7 +647,14 @@ void Ida::IDACore()
       if (writeEventOutput)
       {
         // If we want to write the event-results, we should evaluate the whole system again
-        _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+        if(_dimAE>0)
+		{
+			 _continuous_system->evaluateDAE(IContinuous::CONTINUOUS);
+        }
+		else
+		{
+		    _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+		}
         writeToFile(0, _tCurrent, _h);
       }
 
@@ -581,7 +675,16 @@ void Ida::IDACore()
     {
       _time_system->setTime(_tEnd);
       _continuous_system->setContinuousStates(NV_DATA_S(_CV_y));
-      _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+	  if(_dimAE>0)
+	  {
+	    _mixed_system->setAlgebraicDAEVars(NV_DATA_S(_CV_y)+_dimStates);
+	    _continuous_system->setStateDerivatives(NV_DATA_S(_CV_yp));
+		_continuous_system->evaluateDAE(IContinuous::CONTINUOUS);
+      }
+	  else
+	  {
+		_continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+      }
       if(writeOutput)
          writeToFile(0, _tEnd, _h);
 
@@ -630,7 +733,17 @@ void Ida::writeIDAOutput(const double &time, const double &h, const int &stp)
         _idid = IDAGetDky(_idaMem, _tLastWrite, 0, _CV_yWrite);
         _time_system->setTime(_tLastWrite);
         _continuous_system->setContinuousStates(NV_DATA_S(_CV_yWrite));
-        _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+		if(_dimAE>0)
+		{
+		   _mixed_system->setAlgebraicDAEVars(NV_DATA_S(_CV_y)+_dimStates);
+		   _idid = IDAGetDky(_idaMem, _tLastWrite, 1, _CV_ypWrite);
+		   _continuous_system->setStateDerivatives(NV_DATA_S(_CV_ypWrite));
+		   _continuous_system->evaluateDAE(IContinuous::CONTINUOUS);
+        }
+		else
+		{
+			_continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+		}
         #ifdef RUNTIME_PROFILING
         if(MeasureTime::getInstance() != NULL)
         {
@@ -650,17 +763,31 @@ void Ida::writeIDAOutput(const double &time, const double &h, const int &stp)
       if (_bWritten)
       {
         _time_system->setTime(time);
-        _continuous_system->setContinuousStates(_z);
-        _continuous_system->setRHS(oldValues);
+        _continuous_system->setContinuousStates(_y);
+        _continuous_system->setStateDerivatives(oldValues);
+		if(_dimAE>0)
+		{
+		   _mixed_system->setAlgebraicDAEVars(_y+_dimStates);
+		}
         delete[] oldValues;
-        //_continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+
       }
       else if (time == _tEnd && _tLastWrite != time)
       {
         _idid = IDAGetDky(_idaMem, time, 0, _CV_y);
+		_idid = IDAGetDky(_idaMem, time, 1, _CV_yp);
         _time_system->setTime(time);
         _continuous_system->setContinuousStates(NV_DATA_S(_CV_y));
-        _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+		if(_dimAE>0)
+		{
+		   _mixed_system->setAlgebraicDAEVars(NV_DATA_S(_CV_y)+_dimStates);
+		   _continuous_system->setStateDerivatives(NV_DATA_S(_CV_yp));
+		   _continuous_system->evaluateDAE(IContinuous::CONTINUOUS);
+        }
+		else
+		{
+		   _continuous_system->evaluateAll(IContinuous::CONTINUOUS);
+		}
         #ifdef RUNTIME_PROFILING
         if(MeasureTime::getInstance() != NULL)
         {
@@ -687,7 +814,7 @@ bool Ida::stateSelection()
 {
   return SolverDefaultImplementation::stateSelection();
 }
-int Ida::calcFunction(const double& time, const double* y, double* f)
+int Ida::calcFunction(const double& time, const double* y, double *yp,double* res)
 {
   #ifdef RUNTIME_PROFILING
   MEASURETIME_REGION_DEFINE(idaCalcFunctionHandler, "IDACalcFunction");
@@ -700,10 +827,26 @@ int Ida::calcFunction(const double& time, const double* y, double* f)
   int returnValue = 0;
   try
   {
-    _time_system->setTime(time);
-    _continuous_system->setContinuousStates(y);
-    _continuous_system->evaluateODE(IContinuous::CONTINUOUS);
-    _continuous_system->getRHS(f);
+    if(_dimAE>0)
+	{
+	   _time_system->setTime(time);
+      _continuous_system->setContinuousStates(y);
+	  _continuous_system->setStateDerivatives(yp);
+	  _mixed_system->setAlgebraicDAEVars(y+_dimStates);
+	  _continuous_system->evaluateDAE(IContinuous::CONTINUOUS);
+	  _mixed_system->getResidual(res);
+
+	}
+	else
+	{
+	  _time_system->setTime(time);
+      _continuous_system->setContinuousStates(y);
+      _continuous_system->evaluateODE(IContinuous::CONTINUOUS);
+      _continuous_system->getRHS(res);
+	  for(size_t i(0); i<_dimSys; ++i)
+		   res[i]-=yp[i];
+
+	}
   }      //workaround until exception can be catch from c- libraries
   catch (std::exception& ex)
   {
@@ -722,16 +865,15 @@ int Ida::calcFunction(const double& time, const double* y, double* f)
   return returnValue;
 }
 
-int Ida::CV_fCallback(double t, N_Vector y, N_Vector ydot, N_Vector resval, void *user_data)
+int Ida::rhsFunctionCB(double t, N_Vector y, N_Vector ydot, N_Vector resval, void *user_data)
 {
-  double* ypval=NV_DATA_S(ydot);
-  double* rval=NV_DATA_S(resval);
-  int status = ((Ida*) user_data)->calcFunction(t, NV_DATA_S(y), NV_DATA_S(resval));
-  for(size_t i(0); i<((Ida*) user_data)->_dimSys; ++i) rval[i]-=ypval[i];
+
+  int status = ((Ida*) user_data)->calcFunction(t, NV_DATA_S(y), NV_DATA_S(ydot),NV_DATA_S(resval));
+
   return status;
 }
 
-void Ida::giveZeroVal(const double &t, const double *y, double *zeroValue)
+void Ida::giveZeroVal(const double &t, const double *y,const double *yp,double *zeroValue)
 {
   #ifdef RUNTIME_PROFILING
   MEASURETIME_REGION_DEFINE(idaEvalZeroHandler, "evaluateZeroFuncs");
@@ -743,7 +885,11 @@ void Ida::giveZeroVal(const double &t, const double *y, double *zeroValue)
 
   _time_system->setTime(t);
   _continuous_system->setContinuousStates(y);
-
+   if(_dimAE>0)
+   {
+	 _mixed_system->setAlgebraicDAEVars(y+_dimStates);
+	 _continuous_system->setStateDerivatives(yp);
+   }
   // System aktualisieren
   _continuous_system->evaluateZeroFuncs(IContinuous::DISCRETE);
 
@@ -757,14 +903,14 @@ void Ida::giveZeroVal(const double &t, const double *y, double *zeroValue)
   #endif
 }
 
-int Ida::CV_ZerofCallback(double t, N_Vector y, N_Vector yp, double *zeroval, void *user_data)
+int Ida::zeroFunctionCB(double t, N_Vector y, N_Vector yp, double *zeroval, void *user_data)
 {
-  ((Ida*) user_data)->giveZeroVal(t, NV_DATA_S(y), zeroval);
+  ((Ida*) user_data)->giveZeroVal(t, NV_DATA_S(y),NV_DATA_S(yp), zeroval);
 
   return (0);
 }
 
-int Ida::CV_JCallback(long int N, double t, N_Vector y, N_Vector fy, DlsMat Jac,void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
+int Ida::jacobianFunctionCB(long int N, double t, N_Vector y, N_Vector fy, DlsMat Jac,void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   return ((Ida*) user_data)->calcJacobian(t,N, tmp1, tmp2, tmp3,  NV_DATA_S(y), fy, Jac);
 
@@ -827,7 +973,7 @@ int Ida::calcJacobian(double t, long int N, N_Vector fHelp, N_Vector errorWeight
       }
     }
 
-    calcFunction(t, y, fHelp_data);
+    calcFunction(t, y, fHelp_data,fHelp_data);
 
   for (int k = 0; k < _dimSys; k++)
    {
@@ -887,16 +1033,7 @@ int Ida::calcJacobian(double t, long int N, N_Vector fHelp, N_Vector errorWeight
   return 0;
 }
 
-void Ida::initializeColoredJac()
-{
-  /*_colorOfColumn = new int[_dimSys];
-  _system->getAColorOfColumn( _colorOfColumn, _dimSys);
 
-  _system->getJacobian(_jacobianA);
-  _jacobianANonzeros  = boost::numeric::bindings::traits::spmatrix_num_nonzeros (_jacobianA);
-  _jacobianAIndex     = boost::numeric::bindings::traits::spmatrix_index2_storage(_jacobianA);
-  _jacobianALeadindex = boost::numeric::bindings::traits::spmatrix_index1_storage(_jacobianA);*/
-}
 
 int Ida::reportErrorMessage(ostream& messageStream)
 {
@@ -936,12 +1073,12 @@ void Ida::writeSimulationInfo()
 
   flag = IDAGetNonlinSolvStats(_idaMem, &nni, &ncfn);
 
-  LOGGER_WRITE("Cvode: number steps = " + to_string(nst), LC_SOLV, LL_INFO);
-  LOGGER_WRITE("Cvode: function evaluations 'f' = " + to_string(nfe), LC_SOLV, LL_INFO);
-  LOGGER_WRITE("Cvode: error test failures 'netf' = " + to_string(netfS), LC_SOLV, LL_INFO);
-  LOGGER_WRITE("Cvode: linear solver setups 'nsetups' = " + to_string(nsetups), LC_SOLV, LL_INFO);
-  LOGGER_WRITE("Cvode: nonlinear iterations 'nni' = " + to_string(nni), LC_SOLV, LL_INFO);
-  LOGGER_WRITE("Cvode: convergence failures 'ncfn' = " + to_string(ncfn), LC_SOLV, LL_INFO);
+  LOGGER_WRITE("IDA: number steps = " + to_string(nst), LC_SOLVER, LL_INFO);
+  LOGGER_WRITE("IDA: function evaluations 'f' = " + to_string(nfe), LC_SOLVER, LL_INFO);
+  LOGGER_WRITE("IDA: error test failures 'netf' = " + to_string(netfS), LC_SOLVER, LL_INFO);
+  LOGGER_WRITE("IDA: linear solver setups 'nsetups' = " + to_string(nsetups), LC_SOLVER, LL_INFO);
+  LOGGER_WRITE("IDA: nonlinear iterations 'nni' = " + to_string(nni), LC_SOLVER, LL_INFO);
+  LOGGER_WRITE("IDA: convergence failures 'ncfn' = " + to_string(ncfn), LC_SOLVER, LL_INFO);
 }
 
 int Ida::check_flag(void *flagvalue, const char *funcname, int opt)
@@ -978,3 +1115,14 @@ int Ida::check_flag(void *flagvalue, const char *funcname, int opt)
 
   return (0);
 }
+
+
+void Ida::errOutputIDA(int error_code, const char *module, const char *function,
+    char *msg, void *userData)
+{
+
+  cout << "#### IDA error message #####";
+  cout << " -> error code" << error_code << "in module" << module << " and function " << function;
+  cout << " Message: " << msg;
+
+}
