@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -36,7 +36,6 @@
 #include "Util/Helper.h"
 #include "MainWindow.h"
 #include "Options/OptionsDialog.h"
-#include "Modeling/ModelWidgetContainer.h"
 #include "ShapePropertiesDialog.h"
 #include "Modeling/Commands.h"
 #include "Component/ComponentProperties.h"
@@ -382,6 +381,7 @@ void ShapeAnnotation::setDefaults(ShapeAnnotation *pShapeAnnotation)
   mTextString = pShapeAnnotation->mTextString;
   mFontSize = pShapeAnnotation->mFontSize;
   mFontName = pShapeAnnotation->mFontName;
+  mTextStyles = pShapeAnnotation->mTextStyles;
   mHorizontalAlignment = pShapeAnnotation->mHorizontalAlignment;
   mOriginalFileName = mOriginalFileName;
   mFileName = pShapeAnnotation->mFileName;
@@ -484,6 +484,14 @@ QRectF ShapeAnnotation::getBoundingRect() const
 void ShapeAnnotation::applyLinePattern(QPainter *painter)
 {
   qreal thickness = Utilities::convertMMToPixel(mLineThickness);
+  /* Ticket #4490
+   * The specification doesn't say anything about it.
+   * But just to keep this consist with Dymola set a default line thickness for border patterns raised & sunken.
+   * We need better handling of border patterns.
+   */
+  if (mBorderPattern == StringHandler::BorderRaised || mBorderPattern == StringHandler::BorderSunken) {
+    thickness = Utilities::convertMMToPixel(0.25);
+  }
   QPen pen(mLineColor, thickness, StringHandler::getLinePatternType(mLinePattern), Qt::SquareCap, Qt::MiterJoin);
   /* The specification doesn't say anything about it.
    * But just to keep this consist with Dymola we use Qt::BevelJoin for Line shapes.
@@ -493,16 +501,16 @@ void ShapeAnnotation::applyLinePattern(QPainter *painter)
     pen.setJoinStyle(Qt::BevelJoin);
   }
   /* Ticket #3222
-   * Make all the shapes use cosmetic pens so that they perserve their pen widht when scaled i.e zoomed in/out.
-   * Only shapes with border patterns raised & sunken don't use cosmetic pens. We need better handling of border patterns.
+   * Make all the shapes use cosmetic pens so that they perserve their pen width when scaled i.e zoomed in/out.
    */
-  if (mBorderPattern != StringHandler::BorderRaised && mBorderPattern != StringHandler::BorderSunken) {
-    pen.setCosmetic(true);
-  }
+  pen.setCosmetic(true);
   /* Ticket #2272, Ticket #2268.
    * If thickness is greater than 2 then don't make the pen cosmetic since cosmetic pens don't change the width with respect to zoom.
+   * Use non cosmetic pens for Libraries Browser and shapes inside component when thickness is greater than 2.
    */
-  if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap() && thickness > 2) {
+  if (thickness > 2
+      && ((mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap())
+          || mpParentComponent)) {
     pen.setCosmetic(false);
   }
   // if thickness is greater than 1 pixel then use antialiasing.
@@ -967,11 +975,15 @@ void ShapeAnnotation::adjustGeometries()
   */
 void ShapeAnnotation::setShapeFlags(bool enable)
 {
-  /*
-    Only set the ItemIsMovable & ItemSendsGeometryChanges flags on shape if the class is not a system library class
-    AND shape is not an inherited shape.
-    */
-  if (!mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() && !isInheritedShape()) {
+  /* Only set the ItemIsMovable & ItemSendsGeometryChanges flags on shape if the class is not a system library class
+   * AND shape is not an inherited shape.
+   * AND shape is not a OMS connector i.e., input/output signals of fmu.
+   */
+  if (!mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() && !isInheritedShape()
+      && !(mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType() == LibraryTreeItem::OMS
+           && (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getOMSConnector()
+               || mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getOMSBusConnector()
+               || mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getOMSTLMBusConnector()))) {
     setFlag(QGraphicsItem::ItemIsMovable, enable);
     setFlag(QGraphicsItem::ItemSendsGeometryChanges, enable);
   }
@@ -1632,7 +1644,7 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
 
     menu.addSeparator();
     menu.addAction(mpGraphicsView->getDeleteAction());
-  } else {
+  } else if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::Modelica) {
     menu.addAction(mpShapePropertiesAction);
     menu.addSeparator();
     if (isInheritedShape()) {
@@ -1669,6 +1681,14 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
       menu.addSeparator();
       menu.addAction(mpEditTransitionAction);
     }
+  } else if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::OMS) {
+    BitmapAnnotation *pBitmapAnnotation = dynamic_cast<BitmapAnnotation*>(this);
+    if (pBitmapAnnotation && mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getOMSElement()) {
+      menu.addAction(MainWindow::instance()->getAddOrEditIconAction());
+      menu.addAction(MainWindow::instance()->getDeleteIconAction());
+    } else {
+      return;
+    }
   }
   menu.exec(pEvent->screenPos());
 }
