@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -42,10 +42,12 @@
 #include "Editors/BaseEditor.h"
 #include "Editors/ModelicaEditor.h"
 #include "Editors/CompositeModelEditor.h"
+#include "Editors/OMSimulatorEditor.h"
 #include "Editors/CEditor.h"
 #include "Editors/TextEditor.h"
 #include "Editors/MetaModelicaEditor.h"
 #include "LibraryTreeWidget.h"
+#include "OMSimulator.h"
 
 #include <QGraphicsView>
 #include <QGraphicsScene>
@@ -83,6 +85,7 @@ class GraphicsView : public QGraphicsView
 private:
   StringHandler::ViewType mViewType;
   ModelWidget *mpModelWidget;
+  bool mVisualizationView;
   QRectF mExtentRectangle;
   bool mIsCustomScale;
   bool mAddClassAnnotationNeeded;
@@ -130,13 +133,15 @@ private:
   QAction *mpRotateAntiClockwiseAction;
   QAction *mpFlipHorizontalAction;
   QAction *mpFlipVerticalAction;
-  QAction *mpAttributesAction;
+  QAction *mpSetInitialStateAction;
+  QAction *mpCancelTransitionAction;
 public:
-  GraphicsView(StringHandler::ViewType viewType, ModelWidget *parent);
+  GraphicsView(StringHandler::ViewType viewType, ModelWidget *parent, bool visualizationView = false);
   CoOrdinateSystem mCoOrdinateSystem;
   bool mSkipBackground; /* Do not draw the background rectangle */
   StringHandler::ViewType getViewType() {return mViewType;}
   ModelWidget* getModelWidget() {return mpModelWidget;}
+  bool isVisualizationView() {return mVisualizationView;}
   void setExtentRectangle(qreal x1, qreal y1, qreal x2, qreal y2);
   QRectF getExtentRectangle() {return mExtentRectangle;}
   void setIsCustomScale(bool enable) {mIsCustomScale = enable;}
@@ -198,23 +203,26 @@ public:
   QList<Component*> getInheritedComponentsList() {return mInheritedComponentsList;}
   QList<LineAnnotation*> getConnectionsList() {return mConnectionsList;}
   QList<LineAnnotation*> getInheritedConnectionsList() {return mInheritedConnectionsList;}
-  void addConnectionToClass(LineAnnotation *pConnectionLineAnnotation);
+  bool addConnectionToClass(LineAnnotation *pConnectionLineAnnotation);
   void deleteConnectionFromClass(LineAnnotation *pConnectionLineAnnotation);
   void updateConnectionInClass(LineAnnotation *pConnectionLineAnnotation);
   void addConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.append(pConnectionLineAnnotation);}
   void addInheritedConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.append(pConnectionLineAnnotation);}
   void deleteConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.removeOne(pConnectionLineAnnotation);}
+  void removeConnectionsFromView();
   void deleteInheritedConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.removeOne(pConnectionLineAnnotation);}
   QList<LineAnnotation*> getTransitionsList() {return mTransitionsList;}
   void addTransitionToClass(LineAnnotation *pTransitionLineAnnotation);
   void deleteTransitionFromClass(LineAnnotation *pTransitionLineAnnotation);
   void addTransitionToList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.append(pTransitionLineAnnotation);}
   void deleteTransitionFromList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.removeOne(pTransitionLineAnnotation);}
+  void removeTransitionsFromView();
   QList<LineAnnotation*> getInitialStatesList() {return mInitialStatesList;}
   void addInitialStateToClass(LineAnnotation *pInitialStateLineAnnotation);
   void deleteInitialStateFromClass(LineAnnotation *pInitialStateLineAnnotation);
   void addInitialStateToList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.append(pInitialStateLineAnnotation);}
   void deleteInitialStateFromList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.removeOne(pInitialStateLineAnnotation);}
+  void removeInitialStatesFromView();
   void addShapeToList(ShapeAnnotation *pShape, int index = -1);
   void addInheritedShapeToList(ShapeAnnotation *pShape) {mInheritedShapesList.append(pShape);}
   void deleteShape(ShapeAnnotation *pShapeAnnotation);
@@ -228,6 +236,8 @@ public:
   void removeAllComponents() {mComponentsList.clear();}
   void removeAllShapes() {mShapesList.clear();}
   void removeAllConnections() {mConnectionsList.clear();}
+  void removeAllTransitions() {mTransitionsList.clear();}
+  void removeAllInitialStates() {mInitialStatesList.clear();}
   void createLineShape(QPointF point);
   void createPolygonShape(QPointF point);
   void createRectangleShape(QPointF point);
@@ -236,12 +246,15 @@ public:
   void createBitmapShape(QPointF point);
   QRectF itemsBoundingRect();
   QPointF snapPointToGrid(QPointF point);
-  QPointF movePointByGrid(QPointF point);
+  QPointF movePointByGrid(QPointF point, QPointF origin = QPointF(0, 0), bool useShiftModifier = false);
   QPointF roundPoint(QPointF point);
   bool hasAnnotation();
   void addItem(QGraphicsItem *pGraphicsItem);
   void removeItem(QGraphicsItem *pGraphicsItem);
   void fitInViewInternal();
+  void addSystem(QString name, oms_system_enu_t type);
+  void addSubModel(QString name, QString path);
+  void deleteSubModel(QString name);
 private:
   void createActions();
   bool isClassDroppedOnItself(LibraryTreeItem *pLibraryTreeItem);
@@ -298,6 +311,8 @@ public slots:
   void rotateAntiClockwise();
   void flipHorizontal();
   void flipVertical();
+  void setInitialState();
+  void cancelTransition();
 protected:
   virtual void dragMoveEvent(QDragMoveEvent *event);
   virtual void dropEvent(QDropEvent *event);
@@ -352,6 +367,20 @@ private slots:
   void openLatestNewsItem(QListWidgetItem *pItem);
 };
 
+class UndoCommand;
+class UndoStack : public QUndoStack
+{
+  Q_OBJECT
+public:
+  UndoStack(QObject *parent = 0);
+  void push(UndoCommand *cmd);
+
+  bool isEnabled() {return mEnabled;}
+  void setEnabled(bool enable) {mEnabled = enable;}
+private:
+  bool mEnabled;
+};
+
 class ModelWidgetContainer;
 class ModelicaHighlighter;
 class CompositeModelHighlighter;
@@ -370,7 +399,7 @@ public:
   QToolButton* getDocumentationViewToolButton() {return mpDocumentationViewToolButton;}
   GraphicsView* getDiagramGraphicsView() {return mpDiagramGraphicsView;}
   GraphicsView* getIconGraphicsView() {return mpIconGraphicsView;}
-  QUndoStack* getUndoStack() {return mpUndoStack;}
+  UndoStack* getUndoStack() {return mpUndoStack;}
   BaseEditor* getEditor() {return mpEditor;}
   void setModelClassPathLabel(QString path) {mpModelClassPathLabel->setText(path);}
   void setModelFilePathLabel(QString path) {mpModelFilePathLabel->setText(path);}
@@ -379,6 +408,7 @@ public:
   void removeInheritedClass(LibraryTreeItem *pLibraryTreeItem) {mInheritedClassesList.removeOne(pLibraryTreeItem);}
   void clearInheritedClasses() {mInheritedClassesList.clear();}
   QList<LibraryTreeItem*> getInheritedClassesList() {return mInheritedClassesList;}
+  QList<ComponentInfo*> getComponentsList() {return mComponentsList;}
   QMap<QString, QString> getExtendsModifiersMap(QString extendsClass);
   void fetchExtendsModifiers(QString extendsClass);
   void reDrawModelWidgetInheritedClasses();
@@ -390,16 +420,19 @@ public:
   void loadComponents();
   void loadDiagramView();
   void loadConnections();
+  void getModelConnections();
   void createModelWidgetComponents();
+  ShapeAnnotation* drawOMSModelElement();
   Component* getConnectorComponent(Component *pConnectorComponent, QString connectorName);
   void clearGraphicsViews();
   void reDrawModelWidget();
   bool validateText(LibraryTreeItem **pLibraryTreeItem);
   bool modelicaEditorTextChanged(LibraryTreeItem **pLibraryTreeItem);
   void updateChildClasses(LibraryTreeItem *pLibraryTreeItem);
+  bool omsimulatorEditorTextChanged();
   void clearSelection();
   void updateClassAnnotationIfNeeded();
-  void updateModelText(bool updateText = true);
+  void updateModelText();
   void updateModelicaTextManually(QString contents);
   void updateUndoRedoActions();
   void updateDynamicResults(QString resultFileName);
@@ -408,6 +441,10 @@ public:
   bool writeVisualXMLFile(QString fileName, bool canWriteVisualXMLFile = false);
   void beginMacro(const QString &text);
   void endMacro();
+  void updateViewButtonsBasedOnAccess();
+  void associateBusWithConnector(QString busName, QString connectorName);
+  void dissociateBusWithConnector(QString busName, QString connectorName);
+  void associateBusWithConnectors(QString busName);
 private:
   ModelWidgetContainer *mpModelWidgetContainer;
   LibraryTreeItem *mpLibraryTreeItem;
@@ -426,7 +463,7 @@ private:
   GraphicsScene *mpDiagramGraphicsScene;
   GraphicsView *mpIconGraphicsView;
   GraphicsScene *mpIconGraphicsScene;
-  QUndoStack *mpUndoStack;
+  UndoStack *mpUndoStack;
   QUndoView *mpUndoView;
   BaseEditor *mpEditor;
   QStatusBar *mpModelStatusBar;
@@ -453,7 +490,6 @@ private:
   void drawModelDiagramComponents();
   void drawModelInheritedClassConnections(ModelWidget *pModelWidget);
   void removeInheritedClassConnections();
-  void getModelConnections();
   void getModelTransitions();
   void getModelInitialStates();
   void getMetaModelSubModels();
@@ -462,6 +498,12 @@ private:
   QString getCompositeModelName();
   void getCompositeModelSubModels();
   void getCompositeModelConnections();
+  void drawOMSModelIconElements();
+  void drawOMSModelDiagramElements();
+  void drawOMSModelConnections();
+  void associateBusWithConnector(QString busName, QString connectorName, GraphicsView *pGraphicsView);
+  void dissociateBusWithConnector(QString busName, QString connectorName, GraphicsView *pGraphicsView);
+  void associateBusWithConnectors(Component *pBusComponent, GraphicsView *pGraphicsView);
 private slots:
   void showIconView(bool checked);
   void showDiagramView(bool checked);
@@ -485,7 +527,7 @@ class ModelWidgetContainer : public QMdiArea
   Q_OBJECT
 public:
   ModelWidgetContainer(QWidget *pParent = 0);
-  void addModelWidget(ModelWidget *pModelWidget, bool checkPreferedView = true);
+  void addModelWidget(ModelWidget *pModelWidget, bool checkPreferedView = true, StringHandler::ViewType viewType = StringHandler::NoView);
   ModelWidget* getCurrentModelWidget();
   ModelWidget* getModelWidget(const QString &className);
   QMdiSubWindow* getCurrentMdiSubWindow();
@@ -515,6 +557,13 @@ public slots:
   void printModel();
   void showSimulationParams();
   void alignInterfaces();
+  void addSystem();
+  void addOrEditIcon();
+  void deleteIcon();
+  void addConnector();
+  void addBus();
+  void addTLMBus();
+  void addSubModel();
 };
 
 #endif // MODELWIDGETCONTAINER_H
